@#define K=75
load params

// individual variables
@#for k in 0:K
    var C_@{k};
@#endfor

@#for k in 0:K
    var B_@{k};
@#endfor

@#for k in 0:K
    var N_@{k};
@#endfor


// aggregate variables
var theta kappa PI w Div T zR A;

parameters  theta_zl kappa_zl R N beta sig phi pue peu u phiR eps ksi rhoA rhoR Bbar
@#for k in 1:K
 psi_@{k}
@#endfor
;

% the steady state values for endogenous variables, treated as ``parameters''
parameters
@#for k in 0:K
C_@{k}_ss B_@{k}_ss
@#endfor
;

varexo Rshock TFPshock ;

// parameter values
set_param_value('theta_zl',         p.theta_zl);
set_param_value('kappa_zl',         p.kappa_zl);
set_param_value('beta',             p.beta);
set_param_value('sig',              p.sig);
set_param_value('phi',              p.phi);
set_param_value('pue',              p.pue);
set_param_value('peu',              p.peu);
set_param_value('u',                p.u);
set_param_value('phiR',             p.phiR);
set_param_value('eps',              p.eps);
set_param_value('R',                p.Rbar);
set_param_value('N',                p.Nbar*(1-p.u));
set_param_value('ksi',              p.ksi);
set_param_value('rhoA',             p.rhoA);
set_param_value('rhoR',             p.rhoR);
set_param_value('Bbar',             p.Bbar);

@#for k in 1:K
set_param_value('psi_@{k}',         p.psi(@{k}));
@#endfor

@#for k in 0:K
set_param_value('C_@{k}_ss',       0);
set_param_value('B_@{k}_ss',       0);
@#endfor


// Model
model;

// unemployed >= 1 quarter
C_0 = theta - T;
B_0 = 0;
N_0 = 0;

// Employed agents and newly unemployed agents (by employment duration)
@#for k in 1:K-1
C_@{k} + B_@{k}  = w*N_@{k} + B_@{k-1}(-1)*R(-1)/PI + Div/(1-u)  - T;
C_@{k}^(-sig)     = beta * R / PI(+1)*(peu*(C_0(+1) + B_@{k}*R/PI(+1))^(-sig) + (1-peu)*C_@{k+1}(+1)^(-sig));
w*C_@{k}^(-sig)   = kappa*N_@{k}^phi;
@#endfor

C_@{K} + B_@{K}  = w*N_@{K} + B_@{K}(-1)*R(-1)/PI + Div/(1-u)  - T;
C_@{K}^(-sig)     = beta * R / PI(+1)*(peu*(C_0(+1) + B_@{K}*R/PI(+1))^(-sig) + (1-peu)*C_@{K}(+1)^(-sig));
w*C_@{K}^(-sig)   = kappa*N_@{K}^phi;

// Price setting equation (Phillips Curve)
1-eps+eps*w/A = phiR*PI*(PI-1) - phiR*beta*PI(+1)*(PI(+1)-1);

// Aggregate dividend
Div = A*N*(1-phi*(PI-1)^2) - w*N;

// Monetary Policy rule
PI=1;

// Government budget constraint
Bbar*R(-1)/PI = Bbar + T;

// market clearing liquid assets
Bbar  = 
@#for k in 1:K
+ B_@{k}*psi_@{k}
@#endfor
;

// market clearing labor
N  = 
@#for k in 1:K
+ N_@{k}*psi_@{k}
@#endfor
;

// shocks
zR      = 1-rhoR + rhoR*zR(-1) + Rshock;
A       = 1-rhoA+ rhoA*A(-1)  + TFPshock;


end;

// *****************************Initial Value******************************
initval;
theta=theta_zl;
kappa=kappa_zl;
PI = 1;
w = (eps - 1) / eps;
Div = N - w*N;
T=R*Bbar;
C_0 = theta-T;
B_0 = 0;
@#for k in 1:K
    C_@{k} = 1;
    N_@{k} = 1;
    B_@{k} = 0;
@#endfor
zR=1;
A=1;

end;

shocks;
var Rshock;  stderr 0;
var TFPshock;  stderr 0;
end;
//resid;
steady;
