@#define K=75
load params

// individual variables
@#for k in 0:K
    var C_@{k};
@#endfor

@#for k in 0:K
    var B_@{k};
@#endfor

@#for k in 0:K
    var N_@{k};
@#endfor


// aggregate variables
var R N PI w Div T zR A Y borlim Y_obs PI_obs R_obs zbl;

parameters  theta kappa Rbar beta sig phi pue peu u phiR eps ksi ksibl Bbar Nbar rhoA rhoR rhobl sigA sigR sigbl
@#for k in 1:K
 psi_@{k}
@#endfor
;

varexo eA eR ebl;

// parameter values
set_param_value('theta',            p.theta);
set_param_value('kappa',            p.kappa);
set_param_value('beta',             p.beta);
set_param_value('sig',              p.sig);
set_param_value('phi',              p.phi);
set_param_value('pue',              p.pue);
set_param_value('peu',              p.peu);
set_param_value('u',                p.u);
set_param_value('phiR',             p.phiR);
set_param_value('eps',              p.eps);
set_param_value('Rbar',             p.Rbar);
set_param_value('ksi',              p.ksi);
set_param_value('ksibl',            p.ksibl);
set_param_value('rhoA',             p.rhoA);
set_param_value('rhoR',             p.rhoR);
set_param_value('rhobl',            p.rhobl);
set_param_value('sigA',             1);
set_param_value('sigR',             1);
set_param_value('sigbl',            1);
set_param_value('Bbar',             p.Bbar);
set_param_value('Nbar',             p.Nbar);

@#for k in 1:K
set_param_value('psi_@{k}',         p.psi(@{k}));
@#endfor

// Model
model;

// unemployed >= 1 quarter
C_0 = theta - T - borlim(-1)*R(-1)/PI + borlim; // note: borlim==-B_lowerbar;
B_0 = - borlim;
N_0 = 0;

// Employed agents and newly unemployed agents (by employment duration)
@#for k in 1:K-1
C_@{k} + B_@{k}  = w*N_@{k} + B_@{k-1}(-1)*R(-1)/PI + Div/(1-u)  - T;
C_@{k}^(-sig)     = beta * R / PI(+1)*(peu*(theta - T(+1) + B_@{k}*R/PI(+1)+borlim(+1))^(-sig) + (1-peu)*C_@{k+1}(+1)^(-sig));
w*C_@{k}^(-sig)   = kappa*N_@{k}^phi;
@#endfor

C_@{K} + B_@{K}  = w*N_@{K} + B_@{K}(-1)*R(-1)/PI + Div/(1-u)  - T;
C_@{K}^(-sig)     = beta * R / PI(+1)*(peu*(theta - T(+1)  + B_@{K}*R/PI(+1)+borlim(+1))^(-sig) + (1-peu)*C_@{K}(+1)^(-sig));
w*C_@{K}^(-sig)   = kappa*N_@{K}^phi;

// market clearing liquid assets
Bbar  = 
@#for k in 1:K
+ B_@{k}*psi_@{k}
@#endfor
-u*borlim
;

// market clearing labor
N  = 
@#for k in 1:K
+ N_@{k}*psi_@{k}
@#endfor
;

// Price setting equation (Phillips Curve)
1-eps+eps*w/exp(A) = phiR*PI*(PI-1) - phiR*beta*PI(+1)*(PI(+1)-1);

// Aggregate dividend
Div = exp(A)*N*(1-phi*(PI-1)^2) - w*N;

// Monetary Policy rule
R/Rbar = exp(zR)*PI^ksi;

// Government budget constraint
Bbar*R(-1)/PI = Bbar + T;

// borrowing limit
borlim  = ksibl*(PI-1) +zbl;

// shocks
A       = rhoA*A(-1)    +   sigA*eA;
zR      = rhoR*zR(-1)   +   sigR*eR;
zbl     = rhobl*zbl(-1) +   sigbl*ebl;
 
// output (for IRF)
exp(Y) = exp(A)*N;

// define observables
Y_obs=Y-Y(-1);
R_obs=R^4-Rbar^4;
PI_obs=PI^4-1;

end;

// *****************************Initial Value******************************
initval;
borlim=0;
R=Rbar;
N=Nbar;
PI = 1;
w = (eps - 1) / eps;
Div = N - w*N;
T=R*Bbar;
C_0 = theta-T;
N_0 = 0;
B_@{k} = 0;
@#for k in 1:K
    C_@{k} = 1;
    N_@{k} = 0;
    B_@{k} = 0;
@#endfor
zR=0;
A=0;
zbl=0;
Y=log(1-u);
Y_obs=0;
PI_obs=0;
R_obs=0;
end;

shocks;
var eA;  stderr 1;
var eR;  stderr 1;
var ebl;  stderr 1;
end;
resid;

steady;

stoch_simul(order=1,irf=24,nograph,noprint) Y PI R borlim A zR zbl; 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Compute and plot determinacy region%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
iter=0;
options_.qz_criterium = 1+1e-6;
    ksi_vec=linspace(0,1.5,15);
    ksibl_vec=linspace(-25,0,15);
    [ksi_mat,ksibl_mat]=meshgrid(ksi_vec,ksibl_vec);
    Z_plot_total=zeros(size(ksi_mat));
    
    
     info_mat=NaN(size(ksi_mat));
        for ksi_iter=1:length(ksi_vec)
            for ksibl_iter=1:length(ksibl_vec)
                iter=iter+1
                set_param_value('ksi',ksi_mat(ksi_iter,ksibl_iter));
                set_param_value('ksibl',ksibl_mat(ksi_iter,ksibl_iter));
                [dr,info]=resol(0,M_,options_,oo_);
                info_mat(ksi_iter,ksibl_iter)=info(1);
            end
        end
        Z_plot=zeros(size(info_mat));
        Z_plot(info_mat==0)=1;
        figure(1)
        contour(ksi_mat,ksibl_mat,Z_plot,1,'k','Linewidth',1.5)
        xlabel('\xi')
        ylabel('\xi_{bl}')
        title('determinacy region')   
  
 saveas(gcf,'QE_det.fig')       
 saveas(gcf,'QE_det.pdf')