load params

// variables and parameters
var Y PI A zR peu_endo;
varexo eA eR;
parameters beta eps phi sig ksi phiR Rbar rhoA rhoR peu closs zeta;


// parameter values
set_param_value('beta',p.beta);
set_param_value('eps',p.eps);
set_param_value('phi',p.phi);
set_param_value('sig',p.sig);
set_param_value('ksi',p.ksi);
set_param_value('phiR',p.phiR);
set_param_value('Rbar',p.Rbar);
set_param_value('peu',p.peu);
set_param_value('closs',p.closs);
set_param_value('zeta',p.zeta);
set_param_value('rhoA',p.rhoA);
set_param_value('rhoR',p.rhoR);


// Model declaration
model;

// Euler equation
-sig*Y + sig*beta*Rbar*(1-peu)*Y(+1) - beta*Rbar*((1-closs)^(-sig)-1)*peu_endo  = ksi*PI - PI(+1) + zR;

// eu probability
peu_endo=peu*zeta*Y;

// Price setting equation (Phillips Curve)
((eps-1)/phiR)*((phi+sig)*Y-(1+phi)*A) = PI - beta*PI(+1);

// Evolution shocks
A=rhoA*A(-1)+eA;
zR=rhoR*zR(-1)+eR;

end;

initval;
Y=0;
PI=0;
A=0;
zR=0;
end;

shocks;
var eA;  stderr 1;
var eR;  stderr 1;
end;
resid;

check;
steady;
stoch_simul(order=1,irf=24,nograph); 

iter=0;
options_.qz_criterium = 1+1e-6;
    ksi_vec=linspace(0.5,1.5,50);
    zeta_vec=linspace(-50,50,50);
    [ksi_mat,zeta_mat]=meshgrid(ksi_vec,zeta_vec);
    Z_plot_total=zeros(size(ksi_mat));

        info_mat=NaN(size(ksi_mat));
        for ksi_iter=1:length(ksi_vec)
            for zeta_iter=1:length(zeta_vec)
                iter=iter+1
                set_param_value('ksi',ksi_mat(ksi_iter,zeta_iter));
                set_param_value('zeta',zeta_mat(ksi_iter,zeta_iter));
                [dr,info]=resol(0,M_,options_,oo_);
                info_mat(ksi_iter,zeta_iter)=info(1);
            end
        end
        Z_plot=zeros(size(info_mat));
        Z_plot(info_mat==0)=1;
        figure(2)
        contour(ksi_mat,zeta_mat,Z_plot,1,'k','Linewidth',1.5)
        xlabel('Taylor rule coefficient (\xi)')
        ylabel('Endogenous risk coefficient (\zeta)')
        title('determinacy region')   