
%% C. Cantore (2021)
%% Tractable Heterogeneous Agents Models 
%% CIMS Summer School - University of Surrey


%%=============================================================================================
% Reproduces figures in session 4 on liquidity policy based on material from Vincent Sterk
%%=============================================================================================
 




clc
close all
clear


% parameters
p.Bbar=0;               % aggregate amount of liquidity
p.Rbar=(1.01)^(1/4);      % steady-state target interest rate (quarterly) 
p.Nbar=1;                 % steady-state target labor supply
p.beta=0.99;              % subjective discount factor
p.eps=9;                  % elasticity of subst. goods varieties
p.phi=1;                  % inverse Frische elasticity of labor supply
p.sig=1;                  % coefficient of risk aversion 
p.ksi=1.5;                % coefficient Taylor rule (inflation)
p.pdur=4;                 % average price duration (in quarters)
p.calvo=1-1/p.pdur;       % Calvo probability of adjustment
p.phiR=p.calvo*(p.eps-1)/((1-p.calvo)*(1-p.beta*p.calvo)); % implied Rotemberg parameter (exploit first-order equivalence)
p.peu=0.03;               % job loss prob
p.pue=0.6;                % job finding prob 
p.u=p.peu/(p.pue+p.peu);  % unemployment rate
p.rhoA=0.9;               % persistence TFP shock
p.rhoR=0.5;               % persistence monetary policy shock
p.rhobl=0.9;              % persistence monetary policy shock
p.ksibl=0;            % transmission parameter R to borowing limit

% parameters zero-liquidity version
p.kappa=(p.eps-1)/p.eps;  % normalize kappa to ensure Ne=1 --> kappa=w=(eps-1)/eps) (use labour supply condition and Phillips Curve)
p.theta=((1/p.beta/p.Rbar-1+p.peu)/p.peu).^(-1/p.sig);

K=75;                     % truncation cohort (quarters)
% compute population shares cohorts
p.psi(1) = p.u * p.pue;                                                 % population share employed for 1 quarter 
for i=2:K-1
    p.psi(i)=p.psi(i-1) * (1 - p.peu);                                  % population share employed for i quarters
end
p.psi(K)=1-sum(p.psi(1:K-1))-p.u; 
save params p;

dynare model_moderate_liq.mod noclearall;

% p.ksibl=-50;            % solve again for different value of ksibl
% save params p;
% dynare model_moderate_liq.mod noclearall;
% 
% dynare model_moderate_liq_estimate.mod noclearall;


% figure(4)
% subplot(1,3,1)
% plot(Y_ebl','Linewidth',1.5);hold
% title('Output (Y)');ylabel('%');xlabel('quarter')
% subplot(1,3,2)
% plot(PI_ebl','Linewidth',1.5);hold
% title('Inflation (PI)');ylabel('%');xlabel('quarter')
% subplot(1,3,3)
% plot(borlim_ebl','Linewidth',1.5);hold
% title('Borrowing limit ');ylabel('%');xlabel('quarter')
